require 'Clients'
require 'Propositions'
require 'Ressources'
require 'Incidents'
require 'Contrats'
require 'mysql.so'
require 'Config'

# Class DataManager
#
# definition : Import, store and export the various data used by the application.
# This class is the sole interface between the application and the underlying database.

class DataManager
	attr_reader :clients, :propositions, :ressources, :incidents, :contratsExpertise, :contratsSupport, :contratsProduits
	
	# constructor
	def initialize
		@dBase = nil
		@sqlQuery = nil
		@clients = Clients.new
		@propositions = Propositions.new
		@ressources = Ressources.new
		@incidents = Incidents.new
		@contratsExpertise = ContratsExpertise.new
		@contratsSupport = ContratsSupport.new
		@contratsProduits = ContratsProduits.new
	end
	
	# create the database connection
	def createConnection
		begin
			@dBase = Mysql.connect(Config::HOST,Config::USER,Config::PASSWD,Config::DB)
			puts "Database connection opened"
			puts "Server version: " + @dBase.get_server_info
		rescue MysqlError => e
			print "Error code: ", e.errno, "\n"
			print "Error message: ", e.error, "\n"
		end
		@sqlQuery = SqlQuery.new(@dBase)
	end
	
	# closes the database connection
	def closeConnection
		@dBase.close unless @dBase.nil?
		puts "Database connection closed"
	end
	
	# reads database values to fill local data objects
	def populateDataObjects
		refreshClients
		refreshRessources
	end
	
	###############################################################
	#
	# CLIENTS SECTION
	#
	###############################################################
	
	# update the clients object by retrieving the related data from the database
	# returns the number of clients
	def refreshClients
		@clients.clean
		unless @sqlQuery.nil? then
			@sqlQuery.selectClient.each do |row|
				@clients.addClient(row[0],row[1],row[2],row[3],row[4],row[5], row[6], row[7])
			end
		else
			puts "SqlQuery wasn't created : cannot read data from database"
		end
		return @clients.length
	end
	
	# insert a client in the database and refreshes the local clients object
	# we assume that none of the arguments is null
	# we assume that the client, identified by raison_sociale doesn't already exists
	def addClient(raison_sociale, division, departement, adresse, cp, ville, nom_contact, tel_contact)
		unless @sqlQuery.nil? then
			@sqlQuery.insertClient(raison_sociale, division, departement, adresse, cp, ville, nom_contact,tel_contact)
		else
			puts "SqlQuery wasn't created : database update cannot be performed"
		end
		@clients.addClient(raison_sociale, division, departement, adresse, cp, ville, nom_contact, tel_contact) # synchronize local object with DB
	end
	
	# deletes a client from the database and updates the local Clients object accordingly
	def delClient(raison_sociale, div, dep)
		unless @sqlQuery.nil? then
			@sqlQuery.deleteClient(raison_sociale, div, dep)
		else
			puts "SqlQuery wasn't created : database update cannot be performed"
		end
		@clients.delClient(raison_sociale, div, dep)
	end
	
	###############################################################
	#
	# RESOURCES SECTION
	#
	###############################################################
	
	# update the ressources object by retrieving the related data from the database
	# returns the number of ressources
	def refreshRessources
		@ressources.clean
		unless @sqlQuery.nil? then
			@sqlQuery.selectRessource.each do |row|
				@ressources.addRessource(row[0],row[1],row[2],row[3])
			end
		else
			puts "SqlQuery wasn't created : cannot read data from database"
		end
		return @ressources.length
	end
	
	# inserts a client in the database and refreshes the local ressources object
	def addRessource(nom,prenom,fonction,salaire)
		unless @sqlQuery.nil? then
			@sqlQuery.insertRessource(nom,prenom,fonction,salaire)
		else
			puts "SqlQuery wasn't created : database update cannot be performed"
		end
		@ressources.addRessource(nom,prenom,fonction,salaire)
	end
	
	#deletes a client from the database and updates the local ressouces object accordingly
	def delRessource(nom,prenom,fonction)
		unless @sqlQuery.nil? then
			@sqlQuery.deleteRessource(nom, prenom, fonction)
		else
			puts "SqlQuery wasn't created : database update cannot be performed"
		end
		@ressources.delRessource(nom, prenom, fonction)
	end
	
	# returns the various resources functions
	def getFonctions
		return Config::FONCTIONS
	end
	
	###############################################################
	#
	# CONTRACTS SECTION
	#
	###############################################################
	
	def addContratExp(ref,titre,client,commercial,montant,desc,date_deb,date_fin_prevue)
		@contratsExpertise.addContrat(ref,titre,desc,date_deb,date_fin_prevue,date_deb-date_fin_prevue,montant)
		#puts "Add contrat Exp : "+ref+titre+client+commercial+montant.to_s+desc+date_deb.to_s+date_fin_prevue.to_s
	end
	
	def addContratSup(ref,titre,contratExp,client,commercial,montant,date_deb,date_fin,desc)
		#puts "Add contrat Sup : "+ref+titre+contratExp+client+commercial+montant.to_s+date_deb.to_s+date_fin.to_s+desc
	end
	
	def addContratProd(ref,titre,contratExp,client,commercial,montant,marge,designation,date)
		#puts "Add contrat Prod : "+ref+titre+contratExp+client+commercial+montant.to_s+marge.to_s+designation+date.to_s
	end
	
	# returns the various filters for exp contracts triage
	def getTriExp
		return Config::TRI_EXP
	end
	
	# returns the various filters for sup contracts triage
	def getTriSup
		return Config::TRI_SUP
	end
	
	# returns the various filters for prod contracts triage
	def getTriProd
		return Config::TRI_PROD
	end
end


class SqlQuery
	
	# contructor : We assume that dbHander is not null
	def initialize(dbHandler)
		@dBase = dbHandler
	end

	# returns a list of rows containing every client info
	def selectClient
		clients = []
		res = @dBase.query("SELECT raison_sociale, division, departement, adresse, code_postal, ville, nom_contact, tel_contact FROM clients")
		res.each do |row|
			clients.push([row[0],row[1],row[2], row[3], row[4], row[5], row[6], row[7]])
		end
		res.free
		return clients
	end
	
	# We assume that no argument is null
	def insertClient(raison_sociale, division, departement, adresse, cp, ville, nom_contact, tel_contact)
		rsociale = @dBase.escape_string(raison_sociale)
		div = @dBase.escape_string(division)
		dep = @dBase.escape_string(departement)
		addr = @dBase.escape_string(adresse)
		code = @dBase.escape_string(cp)
		lieu = @dBase.escape_string(ville)
		ncontact = @dBase.escape_string(nom_contact)
		tcontact = @dBase.escape_string(tel_contact)
		@dBase.query("INSERT INTO clients (raison_sociale,division,departement,adresse,code_postal,ville,nom_contact,tel_contact) VALUES ('" + rsociale +"','"+div+"','"+dep+"','"+addr+"','"+code+"','"+lieu+ "','"+ncontact+"','"+tcontact+"')")
	end
	
	# deletes from database the client whose raison_sociale is raison_sociale and division is division and department is departement
	def deleteClient(raison_sociale,division,departement)
		rsociale = @dBase.escape_string(raison_sociale)
		div = @dBase.escape_string(division)
		dep = @dBase.escape_string(departement)
		@dBase.query("DELETE FROM clients WHERE raison_sociale='"+rsociale+"' AND division='"+div+"' AND departement='"+dep+"'")
	end
	
	# return a list of rows containing every resource info
	def selectRessource
		ressources=[]
		res = @dBase.query("SELECT nom,prenom,fonction,salaire FROM ressources")
		res.each do |row|
			ressources.push([row[0],row[1],row[2],row[3]])
		end
		res.free
		return ressources
	end
	
	# inserts a new resource in the database
	def insertRessource(nom,prenom,fonction,salaire)
		name=@dBase.escape_string(nom)
		firstname=@dBase.escape_string(prenom)
		function=@dBase.escape_string(fonction)
		@dBase.query("INSERT INTO ressources (nom,prenom,fonction,salaire) VALUES ('"+name+"','"+firstname+"','"+function+"',"+salaire.to_s+")")
	end
	
	# deletes from the database the resource identified by its name and firstname
	def deleteRessource(nom,prenom,fonction)
		name=@dBase.escape_string(nom)
		firstname=@dBase.escape_string(prenom)
		function=@dBase.escape_string(fonction)
		@dBase.query("DELETE FROM ressources WHERE nom='"+name+"' AND prenom='"+firstname+"' AND fonction='"+function+"'")
	end
end